@@ -1,6 +1,7 @@
-use libc::{self, c_char, c_int, c_void};
+use libc::{c_char, c_int, c_void};
 use std::cmp::Ordering;
 use std::ffi::{CStr, CString};
+use std::iter::FusedIterator;
 use std::marker;
 use std::mem;
 use std::ops::Range;
@@ -36,9 +37,9 @@ pub struct TreeIter<'tree> {
 /// A binary indicator of whether a tree walk should be performed in pre-order
 /// or post-order.
 pub enum TreeWalkMode {
-    /// Runs the traversal in pre order.
+    /// Runs the traversal in pre-order.
     PreOrder = 0,
-    /// Runs the traversal in post order.
+    /// Runs the traversal in post-order.
     PostOrder = 1,
 }
 
@@ -94,14 +95,14 @@ impl<'repo> Tree<'repo> {
         }
     }
 
-    /// Traverse the entries in a tree and its subtrees in post or pre order.
+    /// Traverse the entries in a tree and its subtrees in post or pre-order.
     /// The callback function will be run on each node of the tree that's
     /// walked. The return code of this function will determine how the walk
     /// continues.
     ///
-    /// libgit requires that the callback be an integer, where 0 indicates a
+    /// libgit2 requires that the callback be an integer, where 0 indicates a
     /// successful visit, 1 skips the node, and -1 aborts the traversal completely.
-    /// You may opt to use the enum [`TreeWalkResult`](TreeWalkResult) instead.
+    /// You may opt to use the enum [`TreeWalkResult`] instead.
     ///
     /// ```ignore
     /// let mut ct = 0;
@@ -113,7 +114,7 @@ impl<'repo> Tree<'repo> {
     /// assert_eq!(ct, 1);
     /// ```
     ///
-    /// See [libgit documentation][1] for more information.
+    /// See [libgit2 documentation][1] for more information.
     ///
     /// [1]: https://libgit2.org/libgit2/#HEAD/group/tree/git_tree_walk
     pub fn walk<C, T>(&self, mode: TreeWalkMode, mut callback: C) -> Result<(), Error>
@@ -121,10 +122,6 @@ impl<'repo> Tree<'repo> {
         C: FnMut(&str, &TreeEntry<'_>) -> T,
         T: Into<i32>,
     {
-        #[allow(unused)]
-        struct TreeWalkCbData<'a, T> {
-            pub callback: &'a mut TreeWalkCb<'a, T>,
-        }
         unsafe {
             let mut data = TreeWalkCbData {
                 callback: &mut callback,
@@ -208,6 +205,10 @@ impl<'repo> Tree<'repo> {
 
 type TreeWalkCb<'a, T> = dyn FnMut(&str, &TreeEntry<'_>) -> T + 'a;
 
+struct TreeWalkCbData<'a, T> {
+    callback: &'a mut TreeWalkCb<'a, T>,
+}
+
 extern "C" fn treewalk_cb<T: Into<i32>>(
     root: *const c_char,
     entry: *const raw::git_tree_entry,
@@ -219,8 +220,9 @@ extern "C" fn treewalk_cb<T: Into<i32>>(
             _ => return -1,
         };
         let entry = entry_from_raw_const(entry);
-        let payload = payload as *mut &mut TreeWalkCb<'_, T>;
-        (*payload)(root, &entry).into()
+        let payload = &mut *(payload as *mut TreeWalkCbData<'_, T>);
+        let callback = &mut payload.callback;
+        callback(root, &entry).into()
     }) {
         Some(value) => value,
         None => -1,
@@ -395,12 +397,16 @@ impl<'tree> Iterator for TreeIter<'tree> {
     fn size_hint(&self) -> (usize, Option<usize>) {
         self.range.size_hint()
     }
+    fn nth(&mut self, n: usize) -> Option<TreeEntry<'tree>> {
+        self.range.nth(n).and_then(|i| self.tree.get(i))
+    }
 }
 impl<'tree> DoubleEndedIterator for TreeIter<'tree> {
     fn next_back(&mut self) -> Option<TreeEntry<'tree>> {
         self.range.next_back().and_then(|i| self.tree.get(i))
     }
 }
+impl<'tree> FusedIterator for TreeIter<'tree> {}
 impl<'tree> ExactSizeIterator for TreeIter<'tree> {}
 
 #[cfg(test)]
@@ -469,20 +475,41 @@ mod tests {
 
         let tree = repo.find_tree(commit.tree_id()).unwrap();
         assert_eq!(tree.id(), commit.tree_id());
-        assert_eq!(tree.len(), 1);
+        assert_eq!(tree.len(), 8);
 
         for entry in tree_iter(&tree, &repo) {
             println!("iter entry {:?}", entry.name());
         }
     }
 
+    #[test]
+    fn smoke_tree_nth() {
+        let (td, repo) = crate::test::repo_init();
+
+        setup_repo(&td, &repo);
+
+        let head = repo.head().unwrap();
+        let target = head.target().unwrap();
+        let commit = repo.find_commit(target).unwrap();
+
+        let tree = repo.find_tree(commit.tree_id()).unwrap();
+        assert_eq!(tree.id(), commit.tree_id());
+        assert_eq!(tree.len(), 8);
+        let mut it = tree.iter();
+        let e = it.nth(4).unwrap();
+        assert_eq!(e.name(), Some("f4"));
+    }
+
     fn setup_repo(td: &TempDir, repo: &Repository) {
         let mut index = repo.index().unwrap();
-        File::create(&td.path().join("foo"))
-            .unwrap()
-            .write_all(b"foo")
-            .unwrap();
-        index.add_path(Path::new("foo")).unwrap();
+        for n in 0..8 {
+            let name = format!("f{n}");
+            File::create(&td.path().join(&name))
+                .unwrap()
+                .write_all(name.as_bytes())
+                .unwrap();
+            index.add_path(Path::new(&name)).unwrap();
+        }
         let id = index.write_tree().unwrap();
         let sig = repo.signature().unwrap();
         let tree = repo.find_tree(id).unwrap();
@@ -512,14 +539,22 @@ mod tests {
 
         let tree = repo.find_tree(commit.tree_id()).unwrap();
         assert_eq!(tree.id(), commit.tree_id());
-        assert_eq!(tree.len(), 1);
+        assert_eq!(tree.len(), 8);
         {
-            let e1 = tree.get(0).unwrap();
+            let e0 = tree.get(0).unwrap();
+            assert!(e0 == tree.get_id(e0.id()).unwrap());
+            assert!(e0 == tree.get_name("f0").unwrap());
+            assert!(e0 == tree.get_name_bytes(b"f0").unwrap());
+            assert!(e0 == tree.get_path(Path::new("f0")).unwrap());
+            assert_eq!(e0.name(), Some("f0"));
+            e0.to_object(&repo).unwrap();
+
+            let e1 = tree.get(1).unwrap();
             assert!(e1 == tree.get_id(e1.id()).unwrap());
-            assert!(e1 == tree.get_name("foo").unwrap());
-            assert!(e1 == tree.get_name_bytes(b"foo").unwrap());
-            assert!(e1 == tree.get_path(Path::new("foo")).unwrap());
-            assert_eq!(e1.name(), Some("foo"));
+            assert!(e1 == tree.get_name("f1").unwrap());
+            assert!(e1 == tree.get_name_bytes(b"f1").unwrap());
+            assert!(e1 == tree.get_path(Path::new("f1")).unwrap());
+            assert_eq!(e1.name(), Some("f1"));
             e1.to_object(&repo).unwrap();
         }
         tree.into_object();
@@ -548,20 +583,20 @@ mod tests {
 
         let mut ct = 0;
         tree.walk(TreeWalkMode::PreOrder, |_, entry| {
-            assert_eq!(entry.name(), Some("foo"));
+            assert_eq!(entry.name(), Some(format!("f{ct}").as_str()));
             ct += 1;
             0
         })
         .unwrap();
-        assert_eq!(ct, 1);
+        assert_eq!(ct, 8);
 
         let mut ct = 0;
         tree.walk(TreeWalkMode::PreOrder, |_, entry| {
-            assert_eq!(entry.name(), Some("foo"));
+            assert_eq!(entry.name(), Some(format!("f{ct}").as_str()));
             ct += 1;
             TreeWalkResult::Ok
         })
         .unwrap();
-        assert_eq!(ct, 1);
+        assert_eq!(ct, 8);
     }
 }
